package PLS::Parser::Pod::Method;

use strict;
use warnings;

use parent 'PLS::Parser::Pod::Subroutine';

=head1 NAME

PLS::Parser::Pod::Method

=head1 DESCRIPTION

This is a subclass of L<PLS::Parser::Pod::Subroutine>, to distinguish regular subroutines
from methods.

=cut

1;
